=== modified file 'drizzled/definition/cache.cc'
--- drizzled/definition/cache.cc	2011-01-10 23:48:28 +0000
+++ drizzled/definition/cache.cc	2011-01-13 00:29:52 +0000
@@ -34,7 +34,7 @@
 
 namespace definition {
 
-TableShare::shared_ptr Cache::find(const TableIdentifier::Key &key)
+table::instance::Shared::shared_ptr Cache::find(const TableIdentifier::Key &key)
 {
   boost::mutex::scoped_lock scopedLock(_mutex);
 
@@ -44,7 +44,7 @@
     return (*iter).second;
   }
 
-  return TableShare::shared_ptr();
+  return table::instance::Shared::shared_ptr();
 }
 
 void Cache::erase(const TableIdentifier::Key &key)
@@ -54,7 +54,7 @@
   cache.erase(key);
 }
 
-bool Cache::insert(const TableIdentifier::Key &key, TableShare::shared_ptr share)
+bool Cache::insert(const TableIdentifier::Key &key, table::instance::Shared::shared_ptr share)
 {
   boost::mutex::scoped_lock scopedLock(_mutex);
   std::pair<Map::iterator, bool> ret=
@@ -63,7 +63,7 @@
   return ret.second;
 }
 
-void Cache::CopyFrom(drizzled::TableShare::vector &vector)
+void Cache::CopyFrom(drizzled::table::instance::Shared::vector &vector)
 {
   boost::mutex::scoped_lock scopedLock(_mutex);
 

=== modified file 'drizzled/definition/cache.h'
--- drizzled/definition/cache.h	2011-01-10 23:48:28 +0000
+++ drizzled/definition/cache.h	2011-01-13 00:29:52 +0000
@@ -51,17 +51,17 @@
     cache.rehash(arg);
   }
 
-  TableShare::shared_ptr find(const TableIdentifier::Key &identifier);
+  table::instance::Shared::shared_ptr find(const TableIdentifier::Key &identifier);
   void erase(const TableIdentifier::Key &identifier);
-  bool insert(const TableIdentifier::Key &identifier, TableShare::shared_ptr share);
+  bool insert(const TableIdentifier::Key &identifier, table::instance::Shared::shared_ptr share);
 
 protected:
   friend class drizzled::generator::TableDefinitionCache;
 
-  void CopyFrom(TableShare::vector &vector);
+  void CopyFrom(table::instance::Shared::vector &vector);
 
 private:
-  typedef boost::unordered_map< TableIdentifier::Key, TableShare::shared_ptr> Map;
+  typedef boost::unordered_map< TableIdentifier::Key, table::instance::Shared::shared_ptr> Map;
 
   Map cache;
   boost::mutex _mutex;

=== modified file 'drizzled/generator/table_definition_cache.h'
--- drizzled/generator/table_definition_cache.h	2010-11-19 20:03:08 +0000
+++ drizzled/generator/table_definition_cache.h	2011-01-13 00:29:52 +0000
@@ -28,8 +28,8 @@
 
 class TableDefinitionCache
 {
-  drizzled::TableShare::vector local_vector;
-  drizzled::TableShare::vector::iterator iter;
+  drizzled::table::instance::Shared::vector local_vector;
+  drizzled::table::instance::Shared::vector::iterator iter;
 
 public:
 

=== modified file 'drizzled/include.am'
--- drizzled/include.am	2011-01-10 23:48:28 +0000
+++ drizzled/include.am	2011-01-13 00:29:52 +0000
@@ -441,6 +441,7 @@
 			      drizzled/table/concurrent.h \
 			      drizzled/table/instance.h \
 			      drizzled/table/instance/base.h \
+			      drizzled/table/instance/shared.h \
 			      drizzled/table/placeholder.h \
 			      drizzled/table/shell.h \
 			      drizzled/table/singular.h \
@@ -815,6 +816,7 @@
 			   drizzled/table/cache.cc \
 			   drizzled/table/concurrent.cc \
 			   drizzled/table/instance/base.cc \
+			   drizzled/table/instance/shared.cc \
 			   drizzled/table/singular.cc \
 			   drizzled/table/temporary.cc \
 			   drizzled/table/unused.cc \

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2011-01-08 11:27:50 +0000
+++ drizzled/sql_base.cc	2011-01-13 00:29:52 +0000
@@ -118,7 +118,7 @@
 
   table->cursor->close();
   table->db_stat= 0;                            // Mark cursor closed
-  TableShare::release(table->getMutableShare());
+  table::instance::release(table->getMutableShare());
   table->setShare(share);
 }
 

=== modified file 'drizzled/table/cache.cc'
--- drizzled/table/cache.cc	2011-01-10 17:39:07 +0000
+++ drizzled/table/cache.cc	2011-01-13 00:29:52 +0000
@@ -235,7 +235,7 @@
     table::getUnused().cullByVersion();
 
     /* Remove table from table definition cache if it's not in use */
-    TableShare::release(identifier);
+    table::instance::release(identifier);
 
     if (result && (flags & RTFC_WAIT_OTHER_THREAD_FLAG))
     {

=== modified file 'drizzled/table/cache.h'
--- drizzled/table/cache.h	2010-12-18 04:43:40 +0000
+++ drizzled/table/cache.h	2011-01-13 00:29:52 +0000
@@ -23,10 +23,19 @@
 #define DRIZZLED_TABLE_CACHE_H
 
 #include <boost/unordered_map.hpp>
+#include <drizzled/identifier.h>
+#include <drizzled/table/instance.h>
 
 namespace drizzled {
+
+class Session;
+
 namespace table {
 
+namespace instance {
+class Shared;
+} 
+
 class Concurrent;
 
 typedef boost::unordered_multimap< TableIdentifier::Key, Concurrent *> CacheMap;
@@ -57,7 +66,7 @@
   bool areTablesUsed(Table *table, bool wait_for_name_lock);
   void removeSchema(const SchemaIdentifier &schema_identifier);
   bool removeTable(Session *session, TableIdentifier &identifier, uint32_t flags);
-  void release(TableShare *share);
+  void release(table::instance::Shared *share);
   bool insert(table::Concurrent *arg);
 
   boost::mutex &mutex()

=== modified file 'drizzled/table/concurrent.cc'
--- drizzled/table/concurrent.cc	2010-11-21 21:32:33 +0000
+++ drizzled/table/concurrent.cc	2011-01-13 00:29:52 +0000
@@ -28,6 +28,8 @@
 #include "plugin/myisam/myisam.h"
 #include "drizzled/plugin/transactional_storage_engine.h"
 
+#include <drizzled/table/instance.h>
+
 #include "drizzled/table.h"
 
 namespace drizzled
@@ -127,9 +129,9 @@
 
   safe_mutex_assert_owner(table::Cache::singleton().mutex().native_handle());
 retry:
-  if (not (share= TableShare::getShareCreate(session,
-                                             identifier,
-                                             error)))
+  if (not (share= table::instance::Shared::make_shared(session,
+                                                       identifier,
+                                                       error)))
   {
     return 1;
   }
@@ -148,7 +150,7 @@
       share->resetVersion();                        // Mark share as old
       if (discover_retry_count++)               // Retry once
       {
-        TableShare::release(share);
+        table::instance::release(share);
         return 1;
       }
 
@@ -173,11 +175,12 @@
       */
       if (share->getTableCount() != 1)
       {
-        TableShare::release(share);
+        table::instance::release(share);
         return 1;
       }
+
       /* Free share and wait until it's released by all threads */
-      TableShare::release(share);
+      table::instance::release(share);
 
       if (not session->getKilled())
       {
@@ -185,10 +188,11 @@
         session->clear_error();                 // Clear error message
         goto retry;
       }
+
       return 1;
     }
 
-    TableShare::release(share);
+    table::instance::release(share);
 
     return 1;
   }
@@ -204,7 +208,7 @@
   // delete if this happens.
   if (getShare()->getType() == message::Table::STANDARD)
   {
-    TableShare::release(getMutableShare());
+    table::instance::release(getMutableShare());
   }
   else
   {

=== added file 'drizzled/table/instance.h'
--- drizzled/table/instance.h	1970-01-01 00:00:00 +0000
+++ drizzled/table/instance.h	2011-01-13 00:29:52 +0000
@@ -0,0 +1,41 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#ifndef DRIZZLED_TABLE_INSTANCE_H
+#define DRIZZLED_TABLE_INSTANCE_H
+
+#include <drizzled/table/instance/base.h>
+#include <drizzled/table/instance/shared.h>
+
+namespace drizzled
+{
+
+namespace table
+{
+
+namespace instance
+{
+
+} /* namespace instance */
+} /* namespace table */
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_TABLE_INSTANCE_H */

=== modified file 'drizzled/table/instance/base.cc'
--- drizzled/table/instance/base.cc	2011-01-10 23:48:28 +0000
+++ drizzled/table/instance/base.cc	2011-01-13 00:29:52 +0000
@@ -91,147 +91,6 @@
 
 extern size_t table_def_size;
 
-/*****************************************************************************
-  Functions to handle table definition cach (TableShare)
- *****************************************************************************/
-
-/*
-  Mark that we are not using table share anymore.
-
-  SYNOPSIS
-  release()
-  share		Table share
-
-  IMPLEMENTATION
-  If ref_count goes to zero and (we have done a refresh or if we have
-  already too many open table shares) then delete the definition.
-*/
-
-void TableShare::release(TableShare *share)
-{
-  bool to_be_deleted= false;
-  safe_mutex_assert_owner(table::Cache::singleton().mutex().native_handle);
-
-  share->lock();
-  if (!--share->ref_count)
-  {
-    to_be_deleted= true;
-  }
-  share->unlock();
-
-  if (to_be_deleted)
-  {
-    definition::Cache::singleton().erase(share->getCacheKey());
-  }
-}
-
-void TableShare::release(TableShare::shared_ptr &share)
-{
-  bool to_be_deleted= false;
-  safe_mutex_assert_owner(table::Cache::singleton().mutex().native_handle);
-
-  share->lock();
-  if (!--share->ref_count)
-  {
-    to_be_deleted= true;
-  }
-  share->unlock();
-
-  if (to_be_deleted)
-  {
-    definition::Cache::singleton().erase(share->getCacheKey());
-  }
-}
-
-void TableShare::release(const TableIdentifier &identifier)
-{
-  TableShare::shared_ptr share= definition::Cache::singleton().find(identifier.getKey());
-  if (share)
-  {
-    share->version= 0;                          // Mark for delete
-    if (share->ref_count == 0)
-    {
-      definition::Cache::singleton().erase(identifier.getKey());
-    }
-  }
-}
-
-
-static TableShare::shared_ptr foundTableShare(TableShare::shared_ptr share)
-{
-  /*
-    We found an existing table definition. Return it if we didn't get
-    an error when reading the table definition from file.
-  */
-
-  /* We must do a lock to ensure that the structure is initialized */
-  if (share->error)
-  {
-    /* Table definition contained an error */
-    share->open_table_error(share->error, share->open_errno, share->errarg);
-
-    return TableShare::shared_ptr();
-  }
-
-  share->incrementTableCount();
-
-  return share;
-}
-
-/*
-  Get TableShare for a table.
-
-  get_table_share()
-  session			Thread handle
-  table_list		Table that should be opened
-  key			Table cache key
-  key_length		Length of key
-  error			out: Error code from open_table_def()
-
-  IMPLEMENTATION
-  Get a table definition from the table definition cache.
-  If it doesn't exist, create a new from the table definition file.
-
-  NOTES
-  We must have wrlock on table::Cache::singleton().mutex() when we come here
-  (To be changed later)
-
-  RETURN
-  0  Error
-#  Share for table
-*/
-
-TableShare::shared_ptr TableShare::getShareCreate(Session *session, 
-                                                  const TableIdentifier &identifier,
-                                                  int &in_error)
-{
-  TableShare::shared_ptr share;
-
-  in_error= 0;
-
-  /* Read table definition from cache */
-  if ((share= definition::Cache::singleton().find(identifier.getKey())))
-    return foundTableShare(share);
-
-  share.reset(new TableShare(message::Table::STANDARD, identifier));
-  
-  if (share->open_table_def(*session, identifier))
-  {
-    in_error= share->error;
-
-    return TableShare::shared_ptr();
-  }
-  share->ref_count++;				// Mark in use
-  
-  plugin::EventObserver::registerTableEvents(*share);
-
-  bool ret= definition::Cache::singleton().insert(identifier.getKey(), share);
-
-  if (not ret)
-    return TableShare::shared_ptr();
-
-  return share;
-}
 
 static enum_field_types proto_field_type_to_drizzle_type(const message::Table::Field &field)
 {
@@ -395,7 +254,7 @@
   table_proto(NULL),
   storage_engine(NULL),
   tmp_table(type_arg),
-  ref_count(0),
+  _ref_count(0),
   null_bytes(0),
   last_null_bit_pos(0),
   _field_size(0),
@@ -460,7 +319,7 @@
   table_proto(NULL),
   storage_engine(NULL),
   tmp_table(message::Table::INTERNAL),
-  ref_count(0),
+  _ref_count(0),
   null_bytes(0),
   last_null_bit_pos(0),
   _field_size(0),
@@ -535,7 +394,7 @@
   table_proto(NULL),
   storage_engine(NULL),
   tmp_table(identifier.getType()),
-  ref_count(0),
+  _ref_count(0),
   null_bytes(0),
   last_null_bit_pos(0),
   _field_size(0),
@@ -613,7 +472,7 @@
   table_proto(NULL),
   storage_engine(NULL),
   tmp_table(type_arg),
-  ref_count(0),
+  _ref_count(0),
   null_bytes(0),
   last_null_bit_pos(0),
   _field_size(0),
@@ -704,8 +563,6 @@
 
 TableShare::~TableShare() 
 {
-  assert(ref_count == 0);
-
   storage_engine= NULL;
 
   delete table_proto;

=== modified file 'drizzled/table/instance/base.h'
--- drizzled/table/instance/base.h	2011-01-10 23:48:28 +0000
+++ drizzled/table/instance/base.h	2011-01-13 00:29:52 +0000
@@ -38,7 +38,13 @@
 #include "drizzled/message/table.pb.h"
 #include "drizzled/util/string.h"
 
+#include "drizzled/lex_string.h"
+#include "drizzled/key_map.h"
+ 
 #include "drizzled/table/cache.h"
+ 
+#include <drizzled/field.h>
+
 
 namespace drizzled
 {
@@ -54,6 +60,7 @@
 namespace plugin
 {
 class EventObserverList;
+class StorageEngine;
 }
 
 namespace table {
@@ -80,14 +87,13 @@
              const TableIdentifier &identifier,
              char *path_arg= NULL, uint32_t path_length_arg= 0); // Shares for cache
 
-  ~TableShare();
+  virtual ~TableShare();
 
 private:
   /** Category of this table. */
   enum_table_category table_category;
 
 public:
-
   bool isTemporaryCategory() const
   {
     return (table_category == TABLE_CATEGORY_TEMPORARY);
@@ -102,6 +108,7 @@
   typedef std::vector<Field *> Fields;
 private:
   Fields _fields;
+
 public:
   const Fields getFields() const
   {
@@ -207,6 +214,7 @@
                    arg.begin(), ::toupper);
     _keynames.push_back(arg);
   }
+
 public:
   bool doesKeyNameExist(const char *name_arg, uint32_t name_length, uint32_t &position) const
   {
@@ -234,21 +242,16 @@
 private:
   std::vector<TYPELIB> intervals;			/* pointer to interval info */
 
-  boost::mutex mutex;                /* For locking the share  */
-  boost::condition_variable cond;			/* To signal that share is ready */
-
-
-  void lock()
-  {
-    mutex.lock();
-  }
-
-  void unlock()
-  {
-    mutex.unlock();
-  }
-
+public:
+  virtual void lock()
+  { }
+
+  virtual void unlock()
+  { }
+
+private:
   std::vector<unsigned char> default_values;		/* row with default values */
+
 public:
   // @note This needs to be made to be const in the future
   unsigned char *getDefaultValues()
@@ -260,7 +263,7 @@
     default_values.resize(arg);
   }
 
-  const CHARSET_INFO *table_charset; /* Default charset of string fields */
+  const charset_info_st *table_charset; /* Default charset of string fields */
 
   boost::dynamic_bitset<> all_set;
 
@@ -501,21 +504,26 @@
   }
 
 private:
-  uint32_t ref_count;       /* How many Table objects uses this */
+  uint32_t _ref_count;       /* How many Table objects uses this */
 
 public:
   uint32_t getTableCount() const
   {
-    return ref_count;
+    return _ref_count;
   }
 
   void incrementTableCount()
   {
     lock();
-    ref_count++;
+    _ref_count++;
     unlock();
   }
 
+  uint32_t decrementTableCount()
+  {
+    return --_ref_count;
+  }
+
   uint32_t null_bytes;
   uint32_t last_null_bit_pos;
 private:
@@ -649,9 +657,6 @@
 
   void open_table_error(int pass_error, int db_errno, int pass_errarg);
 
-  static void release(TableShare *share);
-  static void release(TableShare::shared_ptr &share);
-  static void release(const TableIdentifier &identifier);
   static TableShare::shared_ptr getShareCreate(Session *session, 
                                                const TableIdentifier &identifier,
                                                int &error);
@@ -682,7 +687,7 @@
                     unsigned char null_bit,
                     uint8_t decimals,
                     enum_field_types field_type,
-                    const CHARSET_INFO * field_charset,
+                    const charset_info_st * field_charset,
                     Field::utype unireg_check,
                     TYPELIB *interval,
                     const char *field_name);
@@ -695,7 +700,7 @@
                     unsigned char null_bit,
                     uint8_t decimals,
                     enum_field_types field_type,
-                    const CHARSET_INFO * field_charset,
+                    const charset_info_st * field_charset,
                     Field::utype unireg_check,
                     TYPELIB *interval,
                     const char *field_name, 

=== added file 'drizzled/table/instance/shared.cc'
--- drizzled/table/instance/shared.cc	1970-01-01 00:00:00 +0000
+++ drizzled/table/instance/shared.cc	2011-01-13 00:29:52 +0000
@@ -0,0 +1,200 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+
+#include <drizzled/table/instance/shared.h>
+#include <drizzled/definition/cache.h>
+#include <drizzled/plugin/event_observer.h>
+
+namespace drizzled
+{
+
+namespace table
+{
+
+namespace instance
+{
+
+Shared::Shared(const TableIdentifier::Type type_arg,
+               const TableIdentifier &identifier,
+               char *path_arg, uint32_t path_length_arg) :
+  TableShare(type_arg, identifier, path_arg, path_length_arg)
+{
+}
+
+Shared::Shared(const TableIdentifier &identifier) :
+  TableShare(identifier, identifier.getKey())
+{
+}
+
+Shared::shared_ptr Shared::foundTableShare(Shared::shared_ptr share)
+{
+  /*
+    We found an existing table definition. Return it if we didn't get
+    an error when reading the table definition from file.
+  */
+  if (share->error)
+  {
+    /* Table definition contained an error */
+    share->open_table_error(share->error, share->open_errno, share->errarg);
+
+    return Shared::shared_ptr();
+  }
+
+  share->incrementTableCount();
+
+  return share;
+}
+
+
+
+/*
+  Get a shared instance for a table.
+
+  get_table_share()
+  session			Thread handle
+  table_list		Table that should be opened
+  key			Table cache key
+  key_length		Length of key
+  error			out: Error code from open_table_def()
+
+  IMPLEMENTATION
+  Get a table definition from the table definition cache.
+  If it doesn't exist, create a new from the table definition file.
+
+  NOTES
+  We must have wrlock on table::Cache::singleton().mutex() when we come here
+  (To be changed later)
+
+  RETURN
+  0  Error
+#  Share for table
+*/
+
+Shared::shared_ptr Shared::make_shared(Session *session, 
+                                       const TableIdentifier &identifier,
+                                       int &in_error)
+{
+  Shared::shared_ptr share;
+
+  in_error= 0;
+
+  /* Read table definition from cache */
+  if ((share= definition::Cache::singleton().find(identifier.getKey())))
+    return foundTableShare(share);
+
+  share.reset(new Shared(message::Table::STANDARD, identifier));
+  
+  if (share->open_table_def(*session, identifier))
+  {
+    in_error= share->error;
+
+    return Shared::shared_ptr();
+  }
+  share->incrementTableCount();				// Mark in use
+  
+  plugin::EventObserver::registerTableEvents(*share);
+
+  bool ret= definition::Cache::singleton().insert(identifier.getKey(), share);
+
+  if (not ret)
+    return Shared::shared_ptr();
+
+  return share;
+}
+
+Shared::~Shared()
+{
+  assert(getTableCount() == 0);
+}
+
+
+/*****************************************************************************
+  Functions to handle table definition cach (TableShare)
+ *****************************************************************************/
+
+/*
+  Mark that we are not using table share anymore.
+
+  SYNOPSIS
+  release()
+  share		Table share
+
+  IMPLEMENTATION
+  If ref_count goes to zero and (we have done a refresh or if we have
+  already too many open table shares) then delete the definition.
+*/
+
+void release(TableShare *share)
+{
+  bool to_be_deleted= false;
+  //safe_mutex_assert_owner(table::Cache::singleton().mutex().native_handle);
+
+  share->lock();
+  if (not share->decrementTableCount())
+  {
+    to_be_deleted= true;
+  }
+  share->unlock();
+
+  if (to_be_deleted)
+  {
+    definition::Cache::singleton().erase(share->getCacheKey());
+  }
+}
+
+void release(TableShare::shared_ptr &share)
+{
+  bool to_be_deleted= false;
+#if 0
+  safe_mutex_assert_owner(table::Cache::singleton().mutex().native_handle);
+#endif
+
+  share->lock();
+  if (not share->decrementTableCount())
+  {
+    to_be_deleted= true;
+  }
+  share->unlock();
+
+  if (to_be_deleted)
+  {
+    definition::Cache::singleton().erase(share->getCacheKey());
+  }
+}
+
+void release(const TableIdentifier &identifier)
+{
+  TableShare::shared_ptr share= definition::Cache::singleton().find(identifier.getKey());
+  if (share)
+  {
+    share->resetVersion(); 
+    if (share->getTableCount() == 0)
+    {
+      definition::Cache::singleton().erase(identifier.getKey());
+    }
+  }
+}
+
+
+} /* namespace instance */
+} /* namespace table */
+} /* namespace drizzled */

=== added file 'drizzled/table/instance/shared.h'
--- drizzled/table/instance/shared.h	1970-01-01 00:00:00 +0000
+++ drizzled/table/instance/shared.h	2011-01-13 00:29:52 +0000
@@ -0,0 +1,88 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Brian Aker
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#ifndef DRIZZLED_TABLE_INSTANCE_SHARED_H
+#define DRIZZLED_TABLE_INSTANCE_SHARED_H
+
+#include <drizzled/table/instance/base.h>
+
+namespace drizzled
+{
+
+class TableShare;
+class TableIdentifier;
+class Session;
+
+namespace table
+{
+
+namespace instance
+{
+
+void release(TableShare *share);
+void release(TableShare::shared_ptr &share);
+void release(const TableIdentifier &identifier);
+
+
+class Shared : public drizzled::TableShare
+{
+  friend void release(TableShare *share);
+  friend void release(TableShare::shared_ptr &share);
+
+private:
+  boost::mutex mutex;                /* For locking the share  */
+
+public:
+  typedef boost::shared_ptr<Shared> shared_ptr;
+  typedef std::vector <shared_ptr> vector;
+
+  Shared(const TableIdentifier::Type type_arg,
+         const TableIdentifier &identifier,
+         char *path_arg= NULL, uint32_t path_length_arg= 0); // Shares for cache
+
+  Shared(const TableIdentifier &identifier); // Used by placeholder
+
+  ~Shared();
+
+
+  void lock()
+  {
+    mutex.lock();
+  }
+
+  void unlock()
+  {
+    mutex.unlock();
+  }
+
+
+  static shared_ptr make_shared(Session *session, 
+                                const TableIdentifier &identifier,
+                                int &in_error);
+
+  static shared_ptr foundTableShare(shared_ptr share);
+};
+
+} /* namespace instance */
+} /* namespace table */
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_TABLE_INSTANCE_SHARED_H */

=== modified file 'drizzled/table/placeholder.h'
--- drizzled/table/placeholder.h	2010-11-04 20:51:35 +0000
+++ drizzled/table/placeholder.h	2011-01-13 00:29:52 +0000
@@ -31,12 +31,12 @@
 
 class Placeholder : public table::Concurrent
 {
-  TableShare private_share;
+  instance::Shared private_share;
 
 public:
   Placeholder(Session *session, TableIdentifier &identifier) :
     table::Concurrent(),
-    private_share(identifier, identifier.getKey())
+    private_share(identifier)
   {
     setShare(&private_share);
     in_use= session;
@@ -51,7 +51,7 @@
 
   void release(void)
   {
-    TableShare::release(getMutableShare());
+    table::instance::release(getMutableShare());
   }
 };
 

